@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998 Free Software Foundation, Inc.                        *
+ * Copyright (c) 1998-2008,2009 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -29,9 +29,10 @@
 /****************************************************************************
  *  Author: Zeyd M. Ben-Halim <zmbenhal@netcom.com> 1992,1995               *
  *     and: Eric S. Raymond <esr@snark.thyrsus.com>                         *
+ *     and: Thomas E. Dickey                        1996-on                 *
+ *     and: Juergen Pfeifer                         2009                    *
  ****************************************************************************/
 
-
 /*
  *	beep.c
  *
@@ -40,9 +41,12 @@
  */
 
 #include <curses.priv.h>
-#include <term.h>	/* beep, flash */
 
-MODULE_ID("$Id: lib_beep.c,v 1.7 1999/10/22 21:39:09 tom Exp $")
+#ifndef CUR
+#define CUR SP_TERMTYPE
+#endif
+
+MODULE_ID("$Id: lib_beep.c,v 1.14 2009/05/23 19:33:20 tom Exp $")
 
 /*
  *	beep()
@@ -52,22 +56,38 @@ MODULE_ID("$Id: lib_beep.c,v 1.7 1999/10/22 21:39:09 tom Exp $")
  *
  */
 
-int beep(void)
+NCURSES_EXPORT(int)
+NCURSES_SP_NAME(beep) (NCURSES_SP_DCL0)
 {
-	int res = ERR;
+    int res = ERR;
 
-	T((T_CALLED("beep()")));
+    T((T_CALLED("beep(%p)"), SP_PARM));
 
-	/* FIXME: should make sure that we are not in altchar mode */
-	if (bell) {
-		TPUTS_TRACE("bell");
-		res = putp(bell);
-		_nc_flush();
-	} else if (flash_screen) {
-		TPUTS_TRACE("flash_screen");
-		res = putp(flash_screen);
-		_nc_flush();
-	}
+#ifdef USE_TERM_DRIVER
+    if (SP_PARM != 0)
+	res = CallDriver_1(SP_PARM, doBeepOrFlash, TRUE);
+#else
+    /* FIXME: should make sure that we are not in altchar mode */
+    if (cur_term == 0) {
+	res = ERR;
+    } else if (bell) {
+	TPUTS_TRACE("bell");
+	res = putp(bell);
+	_nc_flush();
+    } else if (flash_screen) {
+	TPUTS_TRACE("flash_screen");
+	res = putp(flash_screen);
+	_nc_flush();
+    }
+#endif
 
-	returnCode(res);
+    returnCode(res);
 }
+
+#if NCURSES_SP_FUNCS
+NCURSES_EXPORT(int)
+beep(void)
+{
+    return NCURSES_SP_NAME(beep) (CURRENT_SCREEN);
+}
+#endif
