@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998-2005,2006 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2007,2008 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -41,12 +41,47 @@
 
 #include <curses.priv.h>
 
-MODULE_ID("$Id: lib_getch.c,v 1.75 2006/03/04 20:06:09 tom Exp $")
+MODULE_ID("$Id: lib_getch.c,v 1.99 2008/09/20 19:46:13 tom Exp $")
 
 #include <fifo_defs.h>
 
+#if USE_REENTRANT
+#define GetEscdelay(sp) (sp)->_ESCDELAY
+NCURSES_EXPORT(int)
+NCURSES_PUBLIC_VAR(ESCDELAY) (void)
+{
+    return SP ? GetEscdelay(SP) : 1000;
+}
+#else
+#define GetEscdelay(sp) ESCDELAY
 NCURSES_EXPORT_VAR(int)
 ESCDELAY = 1000;		/* max interval betw. chars in funkeys, in millisecs */
+#endif
+
+#if NCURSES_EXT_FUNCS
+NCURSES_EXPORT(int)
+set_escdelay(int value)
+{
+    int code = OK;
+#if USE_REENTRANT
+    if (SP) {
+	SP->_ESCDELAY = value;
+    } else {
+	code = ERR;
+    }
+#else
+    ESCDELAY = value;
+#endif
+    return code;
+}
+#endif
+
+static int
+_nc_use_meta(WINDOW *win)
+{
+    SCREEN *sp = _nc_screen_of(win);
+    return (sp ? sp->_use_meta : 0);
+}
 
 #ifdef NCURSES_WGETCH_EVENTS
 #define TWAIT_MASK 7
@@ -58,20 +93,20 @@ ESCDELAY = 1000;		/* max interval betw. chars in funkeys, in millisecs */
  * Check for mouse activity, returning nonzero if we find any.
  */
 static int
-check_mouse_activity(int delay EVENTLIST_2nd(_nc_eventlist * evl))
+check_mouse_activity(SCREEN *sp, int delay EVENTLIST_2nd(_nc_eventlist * evl))
 {
     int rc;
 
 #if USE_SYSMOUSE
-    if ((SP->_mouse_type == M_SYSMOUSE)
-	&& (SP->_sysmouse_head < SP->_sysmouse_tail)) {
+    if ((sp->_mouse_type == M_SYSMOUSE)
+	&& (sp->_sysmouse_head < sp->_sysmouse_tail)) {
 	return 2;
     }
 #endif
-    rc = _nc_timed_wait(TWAIT_MASK, delay, (int *) 0 EVENTLIST_2nd(evl));
+    rc = _nc_timed_wait(sp, TWAIT_MASK, delay, (int *) 0 EVENTLIST_2nd(evl));
 #if USE_SYSMOUSE
-    if ((SP->_mouse_type == M_SYSMOUSE)
-	&& (SP->_sysmouse_head < SP->_sysmouse_tail)
+    if ((sp->_mouse_type == M_SYSMOUSE)
+	&& (sp->_sysmouse_head < sp->_sysmouse_tail)
 	&& (rc == 0)
 	&& (errno == EINTR)) {
 	rc |= 2;
@@ -81,9 +116,9 @@ check_mouse_activity(int delay EVENTLIST_2nd(_nc_eventlist * evl))
 }
 
 static NCURSES_INLINE int
-fifo_peek(void)
+fifo_peek(SCREEN *sp)
 {
-    int ch = SP->_fifo[peek];
+    int ch = sp->_fifo[peek];
     TR(TRACE_IEVENT, ("peeking at %d", peek));
 
     p_inc();
@@ -91,11 +126,11 @@ fifo_peek(void)
 }
 
 static NCURSES_INLINE int
-fifo_pull(void)
+fifo_pull(SCREEN *sp)
 {
     int ch;
-    ch = SP->_fifo[head];
-    TR(TRACE_IEVENT, ("pulling %s from %d", _tracechar(ch), head));
+    ch = sp->_fifo[head];
+    TR(TRACE_IEVENT, ("pulling %s from %d", _nc_tracechar(sp, ch), head));
 
     if (peek == head) {
 	h_inc();
@@ -104,14 +139,16 @@ fifo_pull(void)
 	h_inc();
 
 #ifdef TRACE
-    if (_nc_tracing & TRACE_IEVENT)
-	_nc_fifo_dump();
+    if (USE_TRACEF(TRACE_IEVENT)) {
+	_nc_fifo_dump(sp);
+	_nc_unlock_global(tracef);
+    }
 #endif
     return ch;
 }
 
 static NCURSES_INLINE int
-fifo_push(EVENTLIST_0th(_nc_eventlist * evl))
+fifo_push(SCREEN *sp EVENTLIST_2nd(_nc_eventlist * evl))
 {
     int n;
     int ch = 0;
@@ -129,47 +166,47 @@ fifo_push(EVENTLIST_0th(_nc_eventlist * evl))
 #ifdef NCURSES_WGETCH_EVENTS
     if (evl
 #if USE_GPM_SUPPORT || USE_EMX_MOUSE || USE_SYSMOUSE
-	|| (SP->_mouse_fd >= 0)
+	|| (sp->_mouse_fd >= 0)
 #endif
 	) {
-	mask = check_mouse_activity(-1 EVENTLIST_2nd(evl));
+	mask = check_mouse_activity(sp, -1 EVENTLIST_2nd(evl));
     } else
 	mask = 0;
 
     if (mask & 4) {
 	T(("fifo_push: ungetch KEY_EVENT"));
-	ungetch(KEY_EVENT);
+	_nc_ungetch(sp, KEY_EVENT);
 	return KEY_EVENT;
     }
 #elif USE_GPM_SUPPORT || USE_EMX_MOUSE || USE_SYSMOUSE
-    if (SP->_mouse_fd >= 0) {
-	mask = check_mouse_activity(-1 EVENTLIST_2nd(evl));
+    if (sp->_mouse_fd >= 0) {
+	mask = check_mouse_activity(sp, -1 EVENTLIST_2nd(evl));
     }
 #endif
 
 #if USE_GPM_SUPPORT || USE_EMX_MOUSE
-    if ((SP->_mouse_fd >= 0) && (mask & 2)) {
-	SP->_mouse_event(SP);
+    if ((sp->_mouse_fd >= 0) && (mask & 2)) {
+	sp->_mouse_event(sp);
 	ch = KEY_MOUSE;
 	n = 1;
     } else
 #endif
 #if USE_SYSMOUSE
-	if ((SP->_mouse_type == M_SYSMOUSE)
-	    && (SP->_sysmouse_head < SP->_sysmouse_tail)) {
-	SP->_mouse_event(SP);
+	if ((sp->_mouse_type == M_SYSMOUSE)
+	    && (sp->_sysmouse_head < sp->_sysmouse_tail)) {
+	sp->_mouse_event(sp);
 	ch = KEY_MOUSE;
 	n = 1;
-    } else if ((SP->_mouse_type == M_SYSMOUSE)
+    } else if ((sp->_mouse_type == M_SYSMOUSE)
 	       && (mask <= 0) && errno == EINTR) {
-	SP->_mouse_event(SP);
+	sp->_mouse_event(sp);
 	ch = KEY_MOUSE;
 	n = 1;
     } else
 #endif
     {				/* Can block... */
 	unsigned char c2 = 0;
-	n = read(SP->_ifd, &c2, 1);
+	n = read(sp->_ifd, &c2, 1);
 	ch = c2;
     }
 
@@ -188,37 +225,89 @@ fifo_push(EVENTLIST_0th(_nc_eventlist * evl))
 #endif
 
     if ((n == -1) || (n == 0)) {
-	TR(TRACE_IEVENT, ("read(%d,&ch,1)=%d, errno=%d", SP->_ifd, n, errno));
+	TR(TRACE_IEVENT, ("read(%d,&ch,1)=%d, errno=%d", sp->_ifd, n, errno));
 	ch = ERR;
     }
     TR(TRACE_IEVENT, ("read %d characters", n));
 
-    SP->_fifo[tail] = ch;
-    SP->_fifohold = 0;
+    sp->_fifo[tail] = ch;
+    sp->_fifohold = 0;
     if (head == -1)
 	head = peek = tail;
     t_inc();
-    TR(TRACE_IEVENT, ("pushed %s at %d", _tracechar(ch), tail));
+    TR(TRACE_IEVENT, ("pushed %s at %d", _nc_tracechar(sp, ch), tail));
 #ifdef TRACE
-    if (_nc_tracing & TRACE_IEVENT)
-	_nc_fifo_dump();
+    if (USE_TRACEF(TRACE_IEVENT)) {
+	_nc_fifo_dump(sp);
+	_nc_unlock_global(tracef);
+    }
 #endif
     return ch;
 }
 
 static NCURSES_INLINE void
-fifo_clear(void)
+fifo_clear(SCREEN *sp)
 {
-    memset(SP->_fifo, 0, sizeof(SP->_fifo));
+    memset(sp->_fifo, 0, sizeof(sp->_fifo));
     head = -1;
     tail = peek = 0;
 }
 
-static int kgetch(EVENTLIST_0th(_nc_eventlist * evl));
+static int kgetch(SCREEN *EVENTLIST_2nd(_nc_eventlist * evl));
 
-#define wgetch_should_refresh(win) (\
-	(is_wintouched(win) || (win->_flags & _HASMOVED)) \
-	&& !(win->_flags & _ISPAD))
+static void
+recur_wrefresh(WINDOW *win)
+{
+#ifdef USE_PTHREADS
+    SCREEN *sp = _nc_screen_of(win);
+    if (_nc_use_pthreads && sp != SP) {
+	SCREEN *save_SP;
+
+	/* temporarily switch to the window's screen to check/refresh */
+	_nc_lock_global(curses);
+	save_SP = SP;
+	_nc_set_screen(sp);
+	recur_wrefresh(win);
+	_nc_set_screen(save_SP);
+	_nc_unlock_global(curses);
+    } else
+#endif
+	if ((is_wintouched(win) || (win->_flags & _HASMOVED))
+	    && !(win->_flags & _ISPAD)) {
+	wrefresh(win);
+    }
+}
+
+static int
+recur_wgetnstr(WINDOW *win, char *buf)
+{
+    SCREEN *sp = _nc_screen_of(win);
+    int rc;
+
+    if (sp != 0) {
+#ifdef USE_PTHREADS
+	if (_nc_use_pthreads && sp != SP) {
+	    SCREEN *save_SP;
+
+	    /* temporarily switch to the window's screen to get cooked input */
+	    _nc_lock_global(curses);
+	    save_SP = SP;
+	    _nc_set_screen(sp);
+	    rc = recur_wgetnstr(win, buf);
+	    _nc_set_screen(save_SP);
+	    _nc_unlock_global(curses);
+	} else
+#endif
+	{
+	    sp->_called_wgetch = TRUE;
+	    rc = wgetnstr(win, buf, MAXCOLUMNS);
+	    sp->_called_wgetch = FALSE;
+	}
+    } else {
+	rc = ERR;
+    }
+    return rc;
+}
 
 NCURSES_EXPORT(int)
 _nc_wgetch(WINDOW *win,
@@ -226,6 +315,7 @@ _nc_wgetch(WINDOW *win,
 	   int use_meta
 	   EVENTLIST_2nd(_nc_eventlist * evl))
 {
+    SCREEN *sp;
     int ch;
 #ifdef NCURSES_WGETCH_EVENTS
     long event_delay = -1;
@@ -234,15 +324,16 @@ _nc_wgetch(WINDOW *win,
     T((T_CALLED("_nc_wgetch(%p)"), win));
 
     *result = 0;
-    if (win == 0 || SP == 0)
+
+    sp = _nc_screen_of(win);
+    if (win == 0 || sp == 0) {
 	returnCode(ERR);
+    }
 
     if (cooked_key_in_fifo()) {
-	if (wgetch_should_refresh(win))
-	    wrefresh(win);
-
-	*result = fifo_pull();
-	returnCode(OK);
+	recur_wrefresh(win);
+	*result = fifo_pull(sp);
+	returnCode(*result >= KEY_MIN ? KEY_CODE_YES : OK);
     }
 #ifdef NCURSES_WGETCH_EVENTS
     if (evl && (evl->count == 0))
@@ -256,53 +347,48 @@ _nc_wgetch(WINDOW *win,
      * the first character to return it.
      */
     if (head == -1 &&
-	!SP->_notty &&
-	!SP->_raw &&
-	!SP->_cbreak &&
-	!SP->_called_wgetch) {
-	char buf[MAXCOLUMNS], *sp;
+	!sp->_notty &&
+	!sp->_raw &&
+	!sp->_cbreak &&
+	!sp->_called_wgetch) {
+	char buf[MAXCOLUMNS], *bufp;
 	int rc;
 
 	TR(TRACE_IEVENT, ("filling queue in cooked mode"));
 
-	SP->_called_wgetch = TRUE;
-	rc = wgetnstr(win, buf, MAXCOLUMNS);
-	SP->_called_wgetch = FALSE;
+	rc = recur_wgetnstr(win, buf);
 
 	/* ungetch in reverse order */
 #ifdef NCURSES_WGETCH_EVENTS
 	if (rc != KEY_EVENT)
 #endif
-	    ungetch('\n');
-	for (sp = buf + strlen(buf); sp > buf; sp--)
-	    ungetch(sp[-1]);
+	    _nc_ungetch(sp, '\n');
+	for (bufp = buf + strlen(buf); bufp > buf; bufp--)
+	    _nc_ungetch(sp, bufp[-1]);
 
 #ifdef NCURSES_WGETCH_EVENTS
 	/* Return it first */
 	if (rc == KEY_EVENT) {
 	    *result = rc;
-	    returnCode(OK);
-	}
+	} else
 #endif
-
-	*result = fifo_pull();
-	returnCode(OK);
+	    *result = fifo_pull(sp);
+	returnCode(*result >= KEY_MIN ? KEY_CODE_YES : OK);
     }
 
-    if (win->_use_keypad != SP->_keypad_on)
-	_nc_keypad(win->_use_keypad);
+    if (win->_use_keypad != sp->_keypad_on)
+	_nc_keypad(sp, win->_use_keypad);
 
-    if (wgetch_should_refresh(win))
-	wrefresh(win);
+    recur_wrefresh(win);
 
-    if (!win->_notimeout && (win->_delay >= 0 || SP->_cbreak > 1)) {
+    if (win->_notimeout || (win->_delay >= 0) || (sp->_cbreak > 1)) {
 	if (head == -1) {	/* fifo is empty */
 	    int delay;
 	    int rc;
 
 	    TR(TRACE_IEVENT, ("timed delay in wgetch()"));
-	    if (SP->_cbreak > 1)
-		delay = (SP->_cbreak - 1) * 100;
+	    if (sp->_cbreak > 1)
+		delay = (sp->_cbreak - 1) * 100;
 	    else
 		delay = win->_delay;
 
@@ -313,16 +399,17 @@ _nc_wgetch(WINDOW *win,
 
 	    TR(TRACE_IEVENT, ("delay is %d milliseconds", delay));
 
-	    rc = check_mouse_activity(delay EVENTLIST_2nd(evl));
+	    rc = check_mouse_activity(sp, delay EVENTLIST_2nd(evl));
 
 #ifdef NCURSES_WGETCH_EVENTS
 	    if (rc & 4) {
 		*result = KEY_EVENT;
-		returnCode(OK);
+		returnCode(KEY_CODE_YES);
 	    }
 #endif
-	    if (!rc)
+	    if (!rc) {
 		returnCode(ERR);
+	    }
 	}
 	/* else go on to read data available */
     }
@@ -343,23 +430,23 @@ _nc_wgetch(WINDOW *win,
 	int rc;
 
 	do {
-	    ch = kgetch(EVENTLIST_1st(evl));
+	    ch = kgetch(sp EVENTLIST_2nd(evl));
 	    if (ch == KEY_MOUSE) {
 		++runcount;
-		if (SP->_mouse_inline(SP))
+		if (sp->_mouse_inline(sp))
 		    break;
 	    }
-	    if (SP->_maxclick < 0)
+	    if (sp->_maxclick < 0)
 		break;
 	} while
 	    (ch == KEY_MOUSE
-	     && (((rc = check_mouse_activity(SP->_maxclick
+	     && (((rc = check_mouse_activity(sp, sp->_maxclick
 					     EVENTLIST_2nd(evl))) != 0
 		  && !(rc & 4))
-		 || !SP->_mouse_parse(runcount)));
+		 || !sp->_mouse_parse(sp, runcount)));
 #ifdef NCURSES_WGETCH_EVENTS
 	if ((rc & 4) && !ch == KEY_EVENT) {
-	    ungetch(ch);
+	    _nc_ungetch(sp, ch);
 	    ch = KEY_EVENT;
 	}
 #endif
@@ -367,28 +454,28 @@ _nc_wgetch(WINDOW *win,
 #ifdef NCURSES_WGETCH_EVENTS
 	    /* mouse event sequence ended by an event, report event */
 	    if (ch == KEY_EVENT) {
-		ungetch(KEY_MOUSE);	/* FIXME This interrupts a gesture... */
+		_nc_ungetch(sp, KEY_MOUSE);	/* FIXME This interrupts a gesture... */
 	    } else
 #endif
 	    {
 		/* mouse event sequence ended by keystroke, store keystroke */
-		ungetch(ch);
+		_nc_ungetch(sp, ch);
 		ch = KEY_MOUSE;
 	    }
 	}
     } else {
 	if (head == -1)
-	    fifo_push(EVENTLIST_1st(evl));
-	ch = fifo_pull();
+	    fifo_push(sp EVENTLIST_2nd(evl));
+	ch = fifo_pull(sp);
     }
 
     if (ch == ERR) {
 #if USE_SIZECHANGE
-	if (SP->_sig_winch) {
-	    _nc_update_screensize();
+	if (_nc_handle_sigwinch(sp)) {
+	    _nc_update_screensize(sp);
 	    /* resizeterm can push KEY_RESIZE */
 	    if (cooked_key_in_fifo()) {
-		*result = fifo_pull();
+		*result = fifo_pull(sp);
 		returnCode(*result >= KEY_MIN ? KEY_CODE_YES : OK);
 	    }
 	}
@@ -414,7 +501,7 @@ _nc_wgetch(WINDOW *win,
      * However, we provide the same visual result as Solaris, moving the
      * cursor to the left.
      */
-    if (SP->_echo && !(win->_flags & _ISPAD)) {
+    if (sp->_echo && !(win->_flags & _ISPAD)) {
 	chtype backup = (ch == KEY_BACKSPACE) ? '\b' : ch;
 	if (backup < KEY_MIN)
 	    wechochar(win, backup);
@@ -423,7 +510,7 @@ _nc_wgetch(WINDOW *win,
     /*
      * Simulate ICRNL mode
      */
-    if ((ch == '\r') && SP->_nl)
+    if ((ch == '\r') && sp->_nl)
 	ch = '\n';
 
     /* Strip 8th-bit if so desired.  We do this only for characters that
@@ -435,7 +522,7 @@ _nc_wgetch(WINDOW *win,
 	if ((ch < KEY_MIN) && (ch & 0x80))
 	    ch &= 0x7f;
 
-    T(("wgetch returning : %s", _tracechar(ch)));
+    T(("wgetch returning : %s", _nc_tracechar(sp, ch)));
 
     *result = ch;
     returnCode(ch >= KEY_MIN ? KEY_CODE_YES : OK);
@@ -451,7 +538,7 @@ wgetch_events(WINDOW *win, _nc_eventlist * evl)
     T((T_CALLED("wgetch_events(%p,%p)"), win, evl));
     code = _nc_wgetch(win,
 		      &value,
-		      SP->_use_meta
+		      _nc_use_meta(win)
 		      EVENTLIST_2nd(evl));
     if (code != ERR)
 	code = value;
@@ -468,7 +555,7 @@ wgetch(WINDOW *win)
     T((T_CALLED("wgetch(%p)"), win));
     code = _nc_wgetch(win,
 		      &value,
-		      (SP ? SP->_use_meta : 0)
+		      _nc_use_meta(win)
 		      EVENTLIST_2nd((_nc_eventlist *) 0));
     if (code != ERR)
 	code = value;
@@ -491,21 +578,21 @@ wgetch(WINDOW *win)
 */
 
 static int
-kgetch(EVENTLIST_0th(_nc_eventlist * evl))
+kgetch(SCREEN *sp EVENTLIST_2nd(_nc_eventlist * evl))
 {
-    struct tries *ptr;
+    TRIES *ptr;
     int ch = 0;
-    int timeleft = ESCDELAY;
+    int timeleft = GetEscdelay(sp);
 
     TR(TRACE_IEVENT, ("kgetch() called"));
 
-    ptr = SP->_keytry;
+    ptr = sp->_keytry;
 
     for (;;) {
-	if (cooked_key_in_fifo() && SP->_fifo[head] >= KEY_MIN) {
+	if (cooked_key_in_fifo() && sp->_fifo[head] >= KEY_MIN) {
 	    break;
 	} else if (!raw_key_in_fifo()) {
-	    ch = fifo_push(EVENTLIST_1st(evl));
+	    ch = fifo_push(sp EVENTLIST_2nd(evl));
 	    if (ch == ERR) {
 		peek = head;	/* the keys stay uninterpreted */
 		return ERR;
@@ -513,12 +600,12 @@ kgetch(EVENTLIST_0th(_nc_eventlist * evl))
 #ifdef NCURSES_WGETCH_EVENTS
 	    else if (ch == KEY_EVENT) {
 		peek = head;	/* the keys stay uninterpreted */
-		return fifo_pull();	/* Remove KEY_EVENT from the queue */
+		return fifo_pull(sp);	/* Remove KEY_EVENT from the queue */
 	    }
 #endif
 	}
 
-	ch = fifo_peek();
+	ch = fifo_peek(sp);
 	if (ch >= KEY_MIN) {
 	    /* If not first in queue, somebody put this key there on purpose in
 	     * emergency.  Consider it higher priority than the unfinished
@@ -530,7 +617,7 @@ kgetch(EVENTLIST_0th(_nc_eventlist * evl))
 	    return ch;
 	}
 
-	TR(TRACE_IEVENT, ("ch: %s", _tracechar((unsigned char) ch)));
+	TR(TRACE_IEVENT, ("ch: %s", _nc_tracechar(sp, (unsigned char) ch)));
 	while ((ptr != NULL) && (ptr->ch != (unsigned char) ch))
 	    ptr = ptr->sibling;
 
@@ -544,7 +631,7 @@ kgetch(EVENTLIST_0th(_nc_eventlist * evl))
 	if (ptr->value != 0) {	/* sequence terminated */
 	    TR(TRACE_IEVENT, ("end of sequence"));
 	    if (peek == tail)
-		fifo_clear();
+		fifo_clear(sp);
 	    else
 		head = peek;
 	    return (ptr->value);
@@ -556,7 +643,7 @@ kgetch(EVENTLIST_0th(_nc_eventlist * evl))
 	    int rc;
 
 	    TR(TRACE_IEVENT, ("waiting for rest of sequence"));
-	    rc = check_mouse_activity(timeleft EVENTLIST_2nd(evl));
+	    rc = check_mouse_activity(sp, timeleft EVENTLIST_2nd(evl));
 #ifdef NCURSES_WGETCH_EVENTS
 	    if (rc & 4) {
 		TR(TRACE_IEVENT, ("interrupted by a user event"));
@@ -571,7 +658,7 @@ kgetch(EVENTLIST_0th(_nc_eventlist * evl))
 	    }
 	}
     }
-    ch = fifo_pull();
+    ch = fifo_pull(sp);
     peek = head;
     return ch;
 }
