@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998-2006,2007 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2007,2008 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -48,7 +48,7 @@
 #include <term.h>		/* clear_screen, cup & friends, cur_term */
 #include <tic.h>
 
-MODULE_ID("$Id: lib_newterm.c,v 1.67 2007/04/21 20:47:32 tom Exp $")
+MODULE_ID("$Id: lib_newterm.c,v 1.73 2008/08/16 21:20:48 Werner.Fink Exp $")
 
 #ifndef ONLCR			/* Allows compilation under the QNX 4.2 OS */
 #define ONLCR 0
@@ -123,31 +123,32 @@ newterm(NCURSES_CONST char *name, FILE *ofp, FILE *ifp)
 {
     int value;
     int errret;
-    int slk_format = _nc_slk_format;
     SCREEN *current;
     SCREEN *result = 0;
+    TERMINAL *its_term;
 
     START_TRACE();
     T((T_CALLED("newterm(\"%s\",%p,%p)"), name, ofp, ifp));
 
+    _nc_init_pthreads();
+    _nc_lock_global(curses);
+
+    current = SP;
+    its_term = (SP ? SP->_term : 0);
+
     /* this loads the capability entry, then sets LINES and COLS */
-    if (setupterm(name, fileno(ofp), &errret) == ERR) {
-	result = 0;
-    } else {
+    if (setupterm(name, fileno(ofp), &errret) != ERR) {
+	int slk_format = _nc_globals.slk_format;
+
 	/*
 	 * This actually allocates the screen structure, and saves the original
 	 * terminal settings.
 	 */
-	current = SP;
 	_nc_set_screen(0);
 
 	/* allow user to set maximum escape delay from the environment */
 	if ((value = _nc_getenv_num("ESCDELAY")) >= 0) {
-#if USE_REENTRANT
-	    SP->_ESCDELAY = value;
-#else
-	    ESCDELAY = value;
-#endif
+	    set_escdelay(value);
 	}
 
 	if (_nc_setupscreen(LINES,
@@ -158,6 +159,19 @@ newterm(NCURSES_CONST char *name, FILE *ofp, FILE *ifp)
 	    _nc_set_screen(current);
 	    result = 0;
 	} else {
+	    assert(SP != 0);
+	    /*
+	     * In setupterm() we did a set_curterm(), but it was before we set
+	     * SP.  So the "current" screen's terminal pointer was overwritten
+	     * with a different terminal.  Later, in _nc_setupscreen(), we set
+	     * SP and the terminal pointer in the new screen.
+	     *
+	     * Restore the terminal-pointer for the pre-existing screen, if
+	     * any.
+	     */
+	    if (current)
+		current->_term = its_term;
+
 	    /* if the terminal type has real soft labels, set those up */
 	    if (slk_format && num_labels > 0 && SLK_STDFMT(slk_format))
 		_nc_slk_initialize(stdscr, COLS);
@@ -216,5 +230,6 @@ newterm(NCURSES_CONST char *name, FILE *ofp, FILE *ifp)
 	    result = SP;
 	}
     }
+    _nc_unlock_global(curses);
     returnSP(result);
 }
