@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998,1999,2000 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2001,2006 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -40,9 +40,9 @@
 
 #include <curses.priv.h>
 
-MODULE_ID("$Id: lib_vline.c,v 1.5 2000/04/29 21:14:11 tom Exp $")
+MODULE_ID("$Id: lib_vline.c,v 1.10 2006/03/11 21:52:19 tom Exp $")
 
-int
+NCURSES_EXPORT(int)
 wvline(WINDOW *win, chtype ch, int n)
 {
     int code = ERR;
@@ -52,6 +52,7 @@ wvline(WINDOW *win, chtype ch, int n)
     T((T_CALLED("wvline(%p,%s,%d)"), win, _tracechtype(ch), n));
 
     if (win) {
+	NCURSES_CH_T wch;
 	row = win->_cury;
 	col = win->_curx;
 	end = row + n - 1;
@@ -59,12 +60,14 @@ wvline(WINDOW *win, chtype ch, int n)
 	    end = win->_maxy;
 
 	if (ch == 0)
-	    ch = ACS_VLINE;
-	ch = _nc_render(win, ch);
+	    SetChar2(wch, ACS_VLINE);
+	else
+	    SetChar2(wch, ch);
+	wch = _nc_render(win, wch);
 
 	while (end >= row) {
 	    struct ldat *line = &(win->_line[end]);
-	    line->text[col] = ch;
+	    line->text[col] = wch;
 	    CHANGED_CELL(line, col);
 	    end--;
 	}
