@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998-2003,2004 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2006,2008 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -93,7 +93,7 @@
 #include <ctype.h>
 #include <tic.h>
 
-MODULE_ID("$Id: captoinfo.c,v 1.47 2005/06/04 22:06:43 tom Exp $")
+MODULE_ID("$Id: captoinfo.c,v 1.52 2008/08/16 19:24:51 tom Exp $")
 
 #define MAX_PUSHED	16	/* max # args we can push onto the stack */
 
@@ -137,7 +137,7 @@ save_string(char *d, const char *const s)
     return d + strlen(d);
 }
 
-static inline char *
+static NCURSES_INLINE char *
 save_char(char *s, int c)
 {
     static char temp[2];
@@ -149,7 +149,7 @@ static void
 push(void)
 /* push onstack on to the stack */
 {
-    if (stackptr > MAX_PUSHED)
+    if (stackptr >= MAX_PUSHED)
 	_nc_warning("string too complex to convert");
     else
 	stack[stackptr++] = onstack;
@@ -183,7 +183,7 @@ cvtchar(register const char *sp)
 	case '$':
 	case '\\':
 	case '%':
-	    c = *sp;
+	    c = (unsigned char) (*sp);
 	    len = 2;
 	    break;
 	case '\0':
@@ -201,7 +201,7 @@ cvtchar(register const char *sp)
 	    }
 	    break;
 	default:
-	    c = *sp;
+	    c = (unsigned char) (*sp);
 	    len = 2;
 	    break;
 	}
@@ -211,7 +211,7 @@ cvtchar(register const char *sp)
 	len = 2;
 	break;
     default:
-	c = *sp;
+	c = (unsigned char) (*sp);
 	len = 1;
     }
     if (isgraph(c) && c != ',' && c != '\'' && c != '\\' && c != ':') {
@@ -788,6 +788,11 @@ _nc_infotocap(const char *cap GCC_UNUSED, const char *str, int const parameteriz
 	    }			/* endswitch (*str) */
 	}			/* endelse (*str == '%') */
 
+	/*
+	 * 'str' always points to the end of what was scanned in this step,
+	 * but that may not be the end of the string.
+	 */
+	assert(str != 0);
 	if (*str == '\0')
 	    break;
 
@@ -839,7 +844,9 @@ main(int argc, char *argv[])
 NCURSES_EXPORT(void)
 _nc_captoinfo_leaks(void)
 {
-    FreeAndNull(my_string);
+    if (my_string != 0) {
+	FreeAndNull(my_string);
+    }
     my_length = 0;
 }
 #endif
