@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998-2003,2005 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2008,2009 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -50,7 +50,7 @@
 #define DEBUG(level, params)	/*nothing */
 #endif
 
-MODULE_ID("$Id: comp_hash.c,v 1.28 2005/08/20 19:58:18 tom Exp $")
+MODULE_ID("$Id: comp_hash.c,v 1.45 2009/07/18 20:30:21 tom Exp $")
 
 static int hash_function(const char *);
 
@@ -71,25 +71,27 @@ static int hash_function(const char *);
 
 static void
 _nc_make_hash_table(struct name_table_entry *table,
-		    struct name_table_entry **hash_table)
+		    short *hash_table)
 {
-    int i;
+    short i;
     int hashvalue;
     int collisions = 0;
 
+    for (i = 0; i < HASHTABSIZE; i++) {
+	hash_table[i] = -1;
+    }
     for (i = 0; i < CAPTABSIZE; i++) {
 	hashvalue = hash_function(table[i].nte_name);
 
-	if (hash_table[hashvalue] != (struct name_table_entry *) 0)
+	if (hash_table[hashvalue] >= 0)
 	    collisions++;
 
 	if (hash_table[hashvalue] != 0)
-	    table[i].nte_link = (short) (hash_table[hashvalue] - table);
-	hash_table[hashvalue] = &table[i];
+	    table[i].nte_link = hash_table[hashvalue];
+	hash_table[hashvalue] = i;
     }
 
-    DEBUG(4, ("Hash table complete: %d collisions out of %d entries",
-	      collisions, CAPTABSIZE));
+    printf("/* %d collisions out of %d entries */\n", collisions, CAPTABSIZE);
 }
 #endif
 
@@ -118,30 +120,45 @@ hash_function(const char *string)
     return (int) (sum % HASHTABSIZE);
 }
 
-/*
- *	struct name_table_entry *
- *	find_entry(string)
- *
- *	Finds the entry for the given string in the hash table if present.
- *	Returns a pointer to the entry in the table or 0 if not found.
- *
- */
-
 #ifndef MAIN_PROGRAM
+
+#define SameName(a,b,termcap) (termcap ? !strncmp(a,b,2) : !strcmp(a,b))
+#if 0
+static bool
+same_name(const char *a, const char *b, bool termcap)
+{
+    fprintf(stderr, "compare(%s,%s)\n", a, b);
+    return SameName(a, b, termcap);
+}
+#else
+#define same_name(a,b,termcap) SameName(a,b,termcap)
+#endif
+
+/*
+ * Finds the entry for the given string in the hash table if present.
+ * Returns a pointer to the entry in the table or 0 if not found.
+ */
 NCURSES_EXPORT(struct name_table_entry const *)
 _nc_find_entry(const char *string,
-	       const struct name_table_entry *const *hash_table)
+	       const short *hash_table)
 {
     int hashvalue;
-    struct name_table_entry const *ptr;
+    struct name_table_entry const *ptr = 0;
+    struct name_table_entry const *real_table;
 
     hashvalue = hash_function(string);
 
-    if ((ptr = hash_table[hashvalue]) != 0) {
-	while (strcmp(ptr->nte_name, string) != 0) {
-	    if (ptr->nte_link < 0)
-		return 0;
-	    ptr = ptr->nte_link + hash_table[HASHTABSIZE];
+    if (hash_table[hashvalue] >= 0) {
+	bool termcap = (hash_table != _nc_get_hash_table(FALSE));
+
+	real_table = _nc_get_table(termcap);
+	ptr = real_table + hash_table[hashvalue];
+	while (!same_name(ptr->nte_name, string, termcap)) {
+	    if (ptr->nte_link < 0) {
+		ptr = 0;
+		break;
+	    }
+	    ptr = real_table + (ptr->nte_link + hash_table[HASHTABSIZE]);
 	}
     }
 
@@ -149,29 +166,36 @@ _nc_find_entry(const char *string,
 }
 
 /*
- *	struct name_table_entry *
- *	find_type_entry(string, type, table)
+ * Finds the entry for the given name with the given type in the given table if
+ * present (as distinct from _nc_find_entry, which finds the last entry
+ * regardless of type).
  *
- *	Finds the first entry for the given name with the given type in the
- *	given table if present (as distinct from find_entry, which finds the
- *	the last entry regardless of type).  You can use this if you detect
- *	a name clash.  It's slower, though.  Returns a pointer to the entry
- *	in the table or 0 if not found.
+ * Returns a pointer to the entry in the table or 0 if not found.
  */
-
 NCURSES_EXPORT(struct name_table_entry const *)
 _nc_find_type_entry(const char *string,
 		    int type,
-		    const struct name_table_entry *table)
+		    bool termcap)
 {
-    struct name_table_entry const *ptr;
+    struct name_table_entry const *ptr = NULL;
+    const short *hash_table = _nc_get_hash_table(termcap);
+    int hashvalue = hash_function(string);
 
-    for (ptr = table; ptr < table + CAPTABSIZE; ptr++) {
-	if (ptr->nte_type == type && strcmp(string, ptr->nte_name) == 0)
-	    return (ptr);
+    if (hash_table[hashvalue] >= 0) {
+	const struct name_table_entry *const table = _nc_get_table(termcap);
+
+	ptr = table + hash_table[hashvalue];
+	while (ptr->nte_type != type
+	       || !same_name(ptr->nte_name, string, termcap)) {
+	    if (ptr->nte_link < 0) {
+		ptr = 0;
+		break;
+	    }
+	    ptr = table + (ptr->nte_link + hash_table[HASHTABSIZE]);
+	}
     }
 
-    return ((struct name_table_entry *) NULL);
+    return ptr;
 }
 #endif
 
@@ -209,8 +233,9 @@ parse_columns(char *buffer)
 		if ((s - buffer) > 1
 		    && (*buffer == '"')
 		    && (s[-1] == '"')) {	/* strip the quotes */
-		    buffer++;
+		    assert(s > buffer + 1);
 		    s[-1] = '\0';
+		    buffer++;
 		}
 		list[col] = buffer;
 		col++;
@@ -231,10 +256,10 @@ main(int argc, char **argv)
 {
     struct name_table_entry *name_table = typeCalloc(struct
 						     name_table_entry, CAPTABSIZE);
-    struct name_table_entry **hash_table = typeCalloc(struct name_table_entry
-						      *, HASHTABSIZE);
+    short *hash_table = typeCalloc(short, HASHTABSIZE);
     const char *root_name = "";
     int column = 0;
+    int bigstring = 0;
     int n;
     char buffer[BUFSIZ];
 
@@ -248,11 +273,14 @@ main(int argc, char **argv)
     /* The first argument is the column-number (starting with 0).
      * The second is the root name of the tables to generate.
      */
-    if (argc <= 2
+    if (argc <= 3
 	|| (column = atoi(argv[1])) <= 0
 	|| (column >= MAX_COLUMNS)
-	|| *(root_name = argv[2]) == 0) {
-	fprintf(stderr, "usage: make_hash column root_name\n");
+	|| *(root_name = argv[2]) == 0
+	|| (bigstring = atoi(argv[3])) < 0
+	|| name_table == 0
+	|| hash_table == 0) {
+	fprintf(stderr, "usage: make_hash column root_name bigstring\n");
 	exit(EXIT_FAILURE);
     }
 
@@ -288,36 +316,64 @@ main(int argc, char **argv)
     /*
      * Write the compiled tables to standard output
      */
-    printf("static struct name_table_entry const _nc_%s_table[] =\n",
-	   root_name);
-    printf("{\n");
-    for (n = 0; n < CAPTABSIZE; n++) {
-	sprintf(buffer, "\"%s\"",
-		name_table[n].nte_name);
-	printf("\t{ %15s,\t%10s,\t%3d, %3d }%c\n",
-	       buffer,
-	       typenames[name_table[n].nte_type],
-	       name_table[n].nte_index,
-	       name_table[n].nte_link,
-	       n < CAPTABSIZE - 1 ? ',' : ' ');
-    }
-    printf("};\n\n");
+    if (bigstring) {
+	int len = 0;
+	int nxt;
 
-    printf("const struct name_table_entry * const _nc_%s_hash_table[%d] =\n",
+	printf("static const char %s_names_text[] = \\\n", root_name);
+	for (n = 0; n < CAPTABSIZE; n++) {
+	    nxt = (int) strlen(name_table[n].nte_name) + 5;
+	    if (nxt + len > 72) {
+		printf("\\\n");
+		len = 0;
+	    }
+	    printf("\"%s\\0\" ", name_table[n].nte_name);
+	    len += nxt;
+	}
+	printf(";\n\n");
+
+	len = 0;
+	printf("static name_table_data const %s_names_data[] =\n",
+	       root_name);
+	printf("{\n");
+	for (n = 0; n < CAPTABSIZE; n++) {
+	    printf("\t{ %15d,\t%10s,\t%3d, %3d }%c\n",
+		   len,
+		   typenames[name_table[n].nte_type],
+		   name_table[n].nte_index,
+		   name_table[n].nte_link,
+		   n < CAPTABSIZE - 1 ? ',' : ' ');
+	    len += (int) strlen(name_table[n].nte_name) + 1;
+	}
+	printf("};\n\n");
+	printf("static struct name_table_entry *_nc_%s_table = 0;\n\n", root_name);
+    } else {
+
+	printf("static struct name_table_entry %s _nc_%s_table[] =\n",
+	       bigstring ? "" : "const",
+	       root_name);
+	printf("{\n");
+	for (n = 0; n < CAPTABSIZE; n++) {
+	    sprintf(buffer, "\"%s\"",
+		    name_table[n].nte_name);
+	    printf("\t{ %15s,\t%10s,\t%3d, %3d }%c\n",
+		   buffer,
+		   typenames[name_table[n].nte_type],
+		   name_table[n].nte_index,
+		   name_table[n].nte_link,
+		   n < CAPTABSIZE - 1 ? ',' : ' ');
+	}
+	printf("};\n\n");
+    }
+
+    printf("static const short _nc_%s_hash_table[%d] =\n",
 	   root_name,
 	   HASHTABSIZE + 1);
     printf("{\n");
     for (n = 0; n < HASHTABSIZE; n++) {
-	if (hash_table[n] != 0) {
-	    sprintf(buffer, "_nc_%s_table + %3ld",
-		    root_name,
-		    (long) (hash_table[n] - name_table));
-	} else {
-	    strcpy(buffer, "0");
-	}
-	printf("\t%s,\n", buffer);
+	printf("\t%3d,\n", hash_table[n]);
     }
-    printf("\t_nc_%s_table\t/* base-of-table */\n", root_name);
+    printf("\t0\t/* base-of-table */\n");
     printf("};\n\n");
 
     printf("#if (BOOLCOUNT!=%d)||(NUMCOUNT!=%d)||(STRCOUNT!=%d)\n",
@@ -326,6 +382,7 @@ main(int argc, char **argv)
     printf("#error\t--> numbers of booleans, numbers and/or strings <--\n");
     printf("#endif\n\n");
 
+    free(hash_table);
     return EXIT_SUCCESS;
 }
 #endif
