@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 2006,2007 Free Software Foundation, Inc.                   *
+ * Copyright (c) 2006-2008,2009 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -28,6 +28,7 @@
 
 /****************************************************************************
  *  Author: Thomas E. Dickey                                                *
+ *     and: Juergen Pfeifer                                                 *
  ****************************************************************************/
 
 #include <curses.priv.h>
@@ -37,7 +38,7 @@
 #include <tic.h>
 #include <term_entry.h>
 
-MODULE_ID("$Id: entries.c,v 1.3 2007/02/03 18:51:23 tom Exp $")
+MODULE_ID("$Id: entries.c,v 1.15 2009/07/11 14:42:42 tom Exp $")
 
 /****************************************************************************
  *
@@ -111,22 +112,38 @@ _nc_delink_entry(ENTRY * headp, TERMTYPE *tterm)
 NCURSES_EXPORT(void)
 _nc_leaks_tinfo(void)
 {
+#if NO_LEAKS
     char *s;
+#endif
 
     T((T_CALLED("_nc_free_tinfo()")));
 #if NO_LEAKS
     _nc_free_tparm();
     _nc_tgetent_leaks();
-#endif
+
+    if (TerminalOf(CURRENT_SCREEN) != 0) {
+	del_curterm(TerminalOf(CURRENT_SCREEN));
+    }
+
     _nc_free_entries(_nc_head);
     _nc_get_type(0);
     _nc_first_name(0);
-#if NO_LEAKS
     _nc_keyname_leaks();
+#if BROKEN_LINKER || USE_REENTRANT
+    _nc_names_leaks();
+    _nc_codes_leaks();
+    FreeIfNeeded(_nc_prescreen.real_acs_map);
 #endif
 
     if ((s = _nc_home_terminfo()) != 0)
 	free(s);
+
+#ifdef TRACE
+    trace(0);
+    _nc_trace_buf(-1, 0);
+#endif
+
+#endif /* NO_LEAKS */
     returnVoid;
 }
 
