@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1999-2005,2006 Free Software Foundation, Inc.              *
+ * Copyright (c) 1999-2006,2008 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -36,7 +36,7 @@
 
 #include <term_entry.h>
 
-MODULE_ID("$Id: init_keytry.c,v 1.8 2006/01/21 23:43:28 tom Exp $")
+MODULE_ID("$Id: init_keytry.c,v 1.12 2008/05/24 21:44:51 tom Exp $")
 
 /*
 **      _nc_init_keytry()
@@ -45,6 +45,13 @@ MODULE_ID("$Id: init_keytry.c,v 1.8 2006/01/21 23:43:28 tom Exp $")
 **
 */
 
+/*
+ * Internal entrypoints use SCREEN* parameter to obtain capabilities rather
+ * than cur_term.
+ */
+#undef CUR
+#define CUR (sp->_term)->type.
+
 #if	BROKEN_LINKER
 #undef	_nc_tinfo_fkeys
 #endif
@@ -56,7 +63,7 @@ MODULE_ID("$Id: init_keytry.c,v 1.8 2006/01/21 23:43:28 tom Exp $")
 #endif*/
 
 #if	BROKEN_LINKER
-struct tinfo_fkeys *
+const struct tinfo_fkeys *
 _nc_tinfo_fkeysf(void)
 {
     return _nc_tinfo_fkeys;
@@ -64,21 +71,21 @@ _nc_tinfo_fkeysf(void)
 #endif
 
 NCURSES_EXPORT(void)
-_nc_init_keytry(void)
+_nc_init_keytry(SCREEN *sp)
 {
     size_t n;
 
-    /* The SP->_keytry value is initialized in newterm(), where the SP
+    /* The sp->_keytry value is initialized in newterm(), where the sp
      * structure is created, because we can not tell where keypad() or
      * mouse_activate() (which will call keyok()) are first called.
      */
 
-    if (SP != 0) {
+    if (sp != 0) {
 	for (n = 0; _nc_tinfo_fkeys[n].code; n++) {
 	    if (_nc_tinfo_fkeys[n].offset < STRCOUNT) {
-		_nc_add_to_try(&(SP->_keytry),
-			       CUR Strings[_nc_tinfo_fkeys[n].offset],
-			       _nc_tinfo_fkeys[n].code);
+		(void) _nc_add_to_try(&(sp->_keytry),
+				      CUR Strings[_nc_tinfo_fkeys[n].offset],
+				      _nc_tinfo_fkeys[n].code);
 	    }
 	}
 #if NCURSES_XNAMES
@@ -88,7 +95,7 @@ _nc_init_keytry(void)
 	 * names.
 	 */
 	{
-	    TERMTYPE *tp = &(SP->_term->type);
+	    TERMTYPE *tp = &(sp->_term->type);
 	    for (n = STRCOUNT; n < NUM_STRINGS(tp); ++n) {
 		const char *name = ExtStrname(tp, n, strnames);
 		char *value = tp->Strings[n];
@@ -96,15 +103,15 @@ _nc_init_keytry(void)
 		    && *name == 'k'
 		    && value != 0
 		    && key_defined(value) == 0) {
-		    _nc_add_to_try(&(SP->_keytry),
-				   value,
-				   n - STRCOUNT + KEY_MAX);
+		    (void) _nc_add_to_try(&(sp->_keytry),
+					  value,
+					  n - STRCOUNT + KEY_MAX);
 		}
 	    }
 	}
 #endif
 #ifdef TRACE
-	_nc_trace_tries(SP->_keytry);
+	_nc_trace_tries(sp->_keytry);
 #endif
     }
 }
