@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998,2000 Free Software Foundation, Inc.                   *
+ * Copyright (c) 1998-2003,2007 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -40,7 +40,7 @@
 
 #include <curses.priv.h>
 
-MODULE_ID("$Id: setbuf.c,v 1.7 2000/12/10 02:55:08 tom Exp $")
+MODULE_ID("$Id: setbuf.c,v 1.13 2007/05/12 19:04:02 tom Exp $")
 
 /*
  * If the output file descriptor is connected to a tty (the typical case) it
@@ -98,47 +98,53 @@ MODULE_ID("$Id: setbuf.c,v 1.7 2000/12/10 02:55:08 tom Exp $")
  * buffer.  So we disable this by default (there may yet be a workaround).
  */
 NCURSES_EXPORT(void)
-_nc_set_buffer(FILE * ofp, bool buffered)
+_nc_set_buffer(FILE *ofp, bool buffered)
 {
     /* optional optimization hack -- do before any output to ofp */
 #if HAVE_SETVBUF || HAVE_SETBUFFER
-    unsigned buf_len;
-    char *buf_ptr;
+    if (SP->_buffered != buffered) {
+	unsigned buf_len;
+	char *buf_ptr;
 
-    if (getenv("NCURSES_NO_SETBUF") != 0)
-	return;
-
-    fflush(ofp);
-    if ((SP->_buffered = buffered) != 0) {
-	buf_len = min(LINES * (COLS + 6), 2800);
-	if ((buf_ptr = SP->_setbuf) == 0) {
-	    if ((buf_ptr = typeMalloc(char, buf_len)) == NULL)
-		  return;
-	    SP->_setbuf = buf_ptr;
-	    /* Don't try to free this! */
-	}
-#if !USE_SETBUF_0
-	else
+	if (getenv("NCURSES_NO_SETBUF") != 0)
 	    return;
+
+	fflush(ofp);
+#ifdef __DJGPP__
+	setmode(ofp, O_BINARY);
 #endif
-    } else {
+	if (buffered != 0) {
+	    buf_len = min(LINES * (COLS + 6), 2800);
+	    if ((buf_ptr = SP->_setbuf) == 0) {
+		if ((buf_ptr = typeMalloc(char, buf_len)) == NULL)
+		      return;
+		SP->_setbuf = buf_ptr;
+		/* Don't try to free this! */
+	    }
 #if !USE_SETBUF_0
-	return;
-#else
-	buf_len = 0;
-	buf_ptr = 0;
+	    else
+		return;
 #endif
-    }
+	} else {
+#if !USE_SETBUF_0
+	    return;
+#else
+	    buf_len = 0;
+	    buf_ptr = 0;
+#endif
+	}
 
 #if HAVE_SETVBUF
 #ifdef SETVBUF_REVERSED		/* pre-svr3? */
-    (void) setvbuf(ofp, buf_ptr, buf_len, buf_len ? _IOFBF : _IOLBF);
+	(void) setvbuf(ofp, buf_ptr, buf_len, buf_len ? _IOFBF : _IOLBF);
 #else
-    (void) setvbuf(ofp, buf_ptr, buf_len ? _IOFBF : _IOLBF, buf_len);
+	(void) setvbuf(ofp, buf_ptr, buf_len ? _IOFBF : _IOLBF, buf_len);
 #endif
 #elif HAVE_SETBUFFER
-    (void) setbuffer(ofp, buf_ptr, (int) buf_len);
+	(void) setbuffer(ofp, buf_ptr, (int) buf_len);
 #endif
 
+	SP->_buffered = buffered;
+    }
 #endif /* HAVE_SETVBUF || HAVE_SETBUFFER */
 }
