@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998-2002,2003 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2007,2008 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -27,7 +27,7 @@
  ****************************************************************************/
 
 /****************************************************************************
- *  Author: Thomas E. Dickey <dickey@clark.net> 1997                        *
+ *  Author: Thomas E. Dickey                 1997-on                        *
  ****************************************************************************/
 /*
  *	trace_buf.c - Tracing/Debugging buffers (attributes)
@@ -35,46 +35,48 @@
 
 #include <curses.priv.h>
 
-MODULE_ID("$Id: trace_buf.c,v 1.12 2003/03/15 21:21:36 tom Exp $")
+MODULE_ID("$Id: trace_buf.c,v 1.14 2008/08/03 15:13:56 tom Exp $")
 
-typedef struct {
-    char *text;
-    size_t size;
-} LIST;
+#define MyList _nc_globals.tracebuf_ptr
+#define MySize _nc_globals.tracebuf_used
 
 static char *
 _nc_trace_alloc(int bufnum, size_t want)
 {
     char *result = 0;
-    static LIST *list;
-    static size_t have;
 
     if (bufnum >= 0) {
-	if ((size_t) (bufnum + 1) > have) {
+	if ((size_t) (bufnum + 1) > MySize) {
 	    size_t need = (bufnum + 1) * 2;
-	    if ((list = typeRealloc(LIST, need, list)) == 0)
-		return (0);
-	    while (need > have)
-		list[have++].text = 0;
+	    if ((MyList = typeRealloc(TRACEBUF, need, MyList)) != 0) {
+		while (need > MySize)
+		    MyList[MySize++].text = 0;
+	    }
 	}
 
-	if (list[bufnum].text == 0
-	    || want > list[bufnum].size) {
-	    if ((list[bufnum].text = typeRealloc(char, want, list[bufnum].text))
-		!= 0)
-		  list[bufnum].size = want;
+	if (MyList != 0) {
+	    if (MyList[bufnum].text == 0
+		|| want > MyList[bufnum].size) {
+		MyList[bufnum].text = typeRealloc(char, want, MyList[bufnum].text);
+		if (MyList[bufnum].text != 0)
+		    MyList[bufnum].size = want;
+	    }
+	    result = MyList[bufnum].text;
 	}
-
-	result = list[bufnum].text;
     }
 #if NO_LEAKS
     else {
-	if (have) {
-	    while (have--) {
-		if (list[have].text != 0)
-		    free(list[have].text);
+	if (MySize) {
+	    if (MyList) {
+		while (MySize--) {
+		    if (MyList[MySize].text != 0) {
+			free(MyList[MySize].text);
+		    }
+		}
+		free(MyList);
+		MyList = 0;
 	    }
-	    free(list);
+	    MySize = 0;
 	}
     }
 #endif
@@ -100,10 +102,13 @@ NCURSES_EXPORT(char *)
 _nc_trace_bufcat(int bufnum, const char *value)
 {
     char *buffer = _nc_trace_alloc(bufnum, 0);
-    size_t have = strlen(buffer);
+    if (buffer != 0) {
+	size_t have = strlen(buffer);
 
-    buffer = _nc_trace_alloc(bufnum, 1 + have + strlen(value));
-    (void) strcpy(buffer + have, value);
+	buffer = _nc_trace_alloc(bufnum, 1 + have + strlen(value));
+	if (buffer != 0)
+	    (void) strcpy(buffer + have, value);
 
+    }
     return buffer;
 }
