@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998,2000 Free Software Foundation, Inc.                   *
+ * Copyright (c) 1998-2005,2009 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -29,6 +29,7 @@
 /****************************************************************************
  *  Author: Zeyd M. Ben-Halim <zmbenhal@netcom.com> 1995                    *
  *     and: Eric S. Raymond <esr@snark.thyrsus.com>                         *
+ *     and: Juergen Pfeifer                         1997-1999,2008          *
  ****************************************************************************/
 
 /* p_bottom.c
@@ -36,35 +37,38 @@
  */
 #include "panel.priv.h"
 
-MODULE_ID("$Id: p_bottom.c,v 1.9 2001/02/24 23:46:33 tom Exp $")
+MODULE_ID("$Id: p_bottom.c,v 1.12 2008/11/16 00:19:59 juergen Exp $")
 
 NCURSES_EXPORT(int)
-bottom_panel (PANEL *pan)
+bottom_panel(PANEL * pan)
 {
   int err = OK;
 
-  if (pan) {
+  T((T_CALLED("bottom_panel(%p)"), pan));
+  if (pan)
+    {
+      GetHook(pan);
+      if (!Is_Bottom(pan))
+	{
 
-    if(!Is_Bottom(pan)) {
-      
-      dBug(("--> bottom_panel %s", USER_PTR(pan->user)));
-     
-      HIDE_PANEL(pan,err,OK);
-      assert(_nc_bottom_panel == _nc_stdscr_pseudo_panel);
-      
-      dStack("<lb%d>",1,pan);      
-      
-      pan->below = _nc_bottom_panel;
-      pan->above = _nc_bottom_panel->above;
-      if (pan->above)
-	pan->above->below = pan;
-      _nc_bottom_panel->above = pan;
-      
-      dStack("<lb%d>",9,pan);
+	  dBug(("--> bottom_panel %s", USER_PTR(pan->user)));
+
+	  HIDE_PANEL(pan, err, OK);
+	  assert(_nc_bottom_panel == _nc_stdscr_pseudo_panel);
+
+	  dStack("<lb%d>", 1, pan);
+
+	  pan->below = _nc_bottom_panel;
+	  pan->above = _nc_bottom_panel->above;
+	  if (pan->above)
+	    pan->above->below = pan;
+	  _nc_bottom_panel->above = pan;
+
+	  dStack("<lb%d>", 9, pan);
+	}
     }
-  }
   else
     err = ERR;
-  
-  return(err);
+
+  returnCode(err);
 }
