@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998 Free Software Foundation, Inc.                        *
+ * Copyright (c) 1998-2008,2009 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -29,6 +29,8 @@
 /****************************************************************************
  *  Author: Zeyd M. Ben-Halim <zmbenhal@netcom.com> 1995                    *
  *     and: Eric S. Raymond <esr@snark.thyrsus.com>                         *
+ *     and: Juergen Pfeifer                         1997-1999               *
+ *     and: Thomas E. Dickey                        2000-on                 *
  ****************************************************************************/
 
 /* p_new.c
@@ -36,56 +38,85 @@
  */
 #include "panel.priv.h"
 
-MODULE_ID("$Id: p_new.c,v 1.4 1999/09/29 15:22:32 juergen Exp $")
+MODULE_ID("$Id: p_new.c,v 1.14 2009/04/11 20:43:04 tom Exp $")
+
+#ifdef TRACE
+static char *stdscr_id;
+static char *new_id;
+#endif
 
 /*+-------------------------------------------------------------------------
   Get root (i.e. stdscr's) panel.
   Establish the pseudo panel for stdscr if necessary.
 --------------------------------------------------------------------------*/
-static PANEL*
-root_panel(void)
+static PANEL *
+root_panel(NCURSES_SP_DCL0)
 {
-  if(_nc_stdscr_pseudo_panel == (PANEL*)0)
-    {
-      
-      assert(stdscr && !_nc_bottom_panel && !_nc_top_panel);
-      _nc_stdscr_pseudo_panel = (PANEL*)malloc(sizeof(PANEL));
-      if (_nc_stdscr_pseudo_panel != 0) {
-	PANEL* pan  = _nc_stdscr_pseudo_panel;
-	WINDOW* win = stdscr;
-	pan->win = win;
-	pan->below   = (PANEL*)0;
-	pan->above   = (PANEL*)0;
-#ifdef TRACE
-	pan->user = "stdscr";
-#else
-	pan->user = (void*)0;
+#if NCURSES_SP_FUNCS
+  struct panelhook *ph = NCURSES_SP_NAME(_nc_panelhook) (sp);
+
+#elif NO_LEAKS
+  struct panelhook *ph = _nc_panelhook();
 #endif
-	_nc_bottom_panel = _nc_top_panel = pan;
-      }
+
+  if (_nc_stdscr_pseudo_panel == (PANEL *) 0)
+    {
+
+      assert(SP_PARM && SP_PARM->_stdscr && !_nc_bottom_panel && !_nc_top_panel);
+#if NO_LEAKS
+      ph->destroy = del_panel;
+#endif
+      _nc_stdscr_pseudo_panel = (PANEL *) malloc(sizeof(PANEL));
+      if (_nc_stdscr_pseudo_panel != 0)
+	{
+	  PANEL *pan = _nc_stdscr_pseudo_panel;
+	  WINDOW *win = SP_PARM->_stdscr;
+
+	  pan->win = win;
+	  pan->below = (PANEL *) 0;
+	  pan->above = (PANEL *) 0;
+#ifdef TRACE
+	  if (!stdscr_id)
+	    stdscr_id = strdup("stdscr");
+	  pan->user = stdscr_id;
+#else
+	  pan->user = (void *)0;
+#endif
+	  _nc_bottom_panel = _nc_top_panel = pan;
+	}
     }
   return _nc_stdscr_pseudo_panel;
 }
 
-PANEL *
+NCURSES_EXPORT(PANEL *)
 new_panel(WINDOW *win)
 {
-  PANEL *pan = (PANEL*)0;
+  PANEL *pan = (PANEL *) 0;
 
-  (void)root_panel();
+  GetWindowHook(win);
+
+  T((T_CALLED("new_panel(%p)"), win));
+
+  if (!win)
+    returnPanel(pan);
+
+  if (!_nc_stdscr_pseudo_panel)
+    (void)root_panel(NCURSES_SP_ARG);
   assert(_nc_stdscr_pseudo_panel);
 
-  if (!(win->_flags & _ISPAD) && (pan = (PANEL*)malloc(sizeof(PANEL))))
+  if (!(win->_flags & _ISPAD) && (pan = (PANEL *) malloc(sizeof(PANEL))))
     {
       pan->win = win;
-      pan->above = (PANEL *)0;
-      pan->below = (PANEL *)0;
+      pan->above = (PANEL *) 0;
+      pan->below = (PANEL *) 0;
 #ifdef TRACE
-      pan->user = "new";
+      if (!new_id)
+	new_id = strdup("new");
+      pan->user = new_id;
 #else
       pan->user = (char *)0;
 #endif
       (void)show_panel(pan);
     }
-  return(pan);
+  returnPanel(pan);
 }
