@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998 Free Software Foundation, Inc.                        *
+ * Copyright (c) 1998-2000,2005 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -36,32 +36,37 @@
  */
 #include "panel.priv.h"
 
-MODULE_ID("$Id: p_show.c,v 1.7 1999/11/25 13:49:26 juergen Exp $")
+MODULE_ID("$Id: p_show.c,v 1.12 2009/02/07 23:11:45 tom Exp $")
 
-int
-show_panel(PANEL *pan)
-{ 
-  int err = OK;
+NCURSES_EXPORT(int)
+show_panel(PANEL * pan)
+{
+  int err = ERR;
 
-  if(!pan)
-    return(ERR);
+  T((T_CALLED("show_panel(%p)"), pan));
 
-  if (Is_Top(pan))
-    return(OK);
+  if (pan)
+    {
+      GetHook(pan);
 
-  dBug(("--> show_panel %s", USER_PTR(pan->user)));
+      if (Is_Top(pan))
+	returnCode(OK);
 
-  HIDE_PANEL(pan,err,FALSE);
+      dBug(("--> show_panel %s", USER_PTR(pan->user)));
 
-  dStack("<lt%d>",1,pan);
-  assert(_nc_bottom_panel == _nc_stdscr_pseudo_panel);
+      HIDE_PANEL(pan, err, OK);
 
-  _nc_top_panel->above = pan;
-  pan->below = _nc_top_panel;  
-  pan->above = (PANEL *)0;
-  _nc_top_panel = pan;
-  
-  dStack("<lt%d>",9,pan);
+      dStack("<lt%d>", 1, pan);
+      assert(_nc_bottom_panel == _nc_stdscr_pseudo_panel);
 
-  return(OK);
+      _nc_top_panel->above = pan;
+      pan->below = _nc_top_panel;
+      pan->above = (PANEL *) 0;
+      _nc_top_panel = pan;
+
+      err = OK;
+
+      dStack("<lt%d>", 9, pan);
+    }
+  returnCode(err);
 }
