@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 2007 Free Software Foundation, Inc.                        *
+ * Copyright (c) 2007,2008 Free Software Foundation, Inc.                   *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -26,7 +26,7 @@
  * authorization.                                                           *
  ****************************************************************************/
 /*
- * $Id: demo_panels.c,v 1.26 2007/07/07 22:16:33 tom Exp $
+ * $Id: demo_panels.c,v 1.33 2008/08/04 13:33:48 tom Exp $
  *
  * Demonstrate a variety of functions from the panel library.
  */
@@ -205,16 +205,16 @@ mkpanel(short color, int rows, int cols, int tly, int tlx)
 {
     WINDOW *win;
     PANEL *pan = 0;
-    char *userdata = malloc(3);
+    char *userdata = typeMalloc(char, 3);
 
     if ((win = newwin(rows, cols, tly, tlx)) != 0) {
 	keypad(win, TRUE);
 	if ((pan = new_panel(win)) == 0) {
 	    delwin(win);
 	} else if (use_colors) {
-	    short fg = ((color == COLOR_BLUE)
-			? COLOR_WHITE
-			: COLOR_BLACK);
+	    short fg = (short) ((color == COLOR_BLUE)
+				? COLOR_WHITE
+				: COLOR_BLACK);
 	    short bg = color;
 
 	    init_pair(color, fg, bg);
@@ -234,7 +234,7 @@ my_remove_panel(PANEL ** pans, int which)
     if (pans[which] != 0) {
 	PANEL *pan = pans[which];
 	WINDOW *win = panel_window(pan);
-	char *user = panel_userptr(pan);
+	char *user = (char *) panel_userptr(pan);
 
 	free(user);
 	del_panel(pan);
@@ -244,6 +244,7 @@ my_remove_panel(PANEL ** pans, int which)
     }
 }
 
+#undef MIN
 #define MIN(a,b) ((a) < (b) ? (a) : (b))
 #define ABS(a)   ((a) < 0 ? -(a) : (a))
 
@@ -252,8 +253,8 @@ my_create_panel(PANEL ** pans, int which, FillPanel myFill)
 {
     PANEL *pan = 0;
     int code;
-    int pair = which;
-    short fg = (pair == COLOR_BLUE) ? COLOR_WHITE : COLOR_BLACK;
+    short pair = (short) which;
+    short fg = (short) ((pair == COLOR_BLUE) ? COLOR_WHITE : COLOR_BLACK);
     short bg = pair;
     int x0, y0, x1, y1;
 
@@ -368,7 +369,8 @@ static void
 fill_panel(PANEL * pan)
 {
     WINDOW *win = panel_window(pan);
-    int num = ((const char *) panel_userptr(pan))[1];
+    const char *userptr = (const char *) panel_userptr(pan);
+    int num = (userptr && *userptr) ? userptr[1] : '?';
     int y, x;
 
     wmove(win, 1, 1);
@@ -387,7 +389,8 @@ static void
 fill_unboxed(PANEL * pan)
 {
     WINDOW *win = panel_window(pan);
-    int num = ((const char *) panel_userptr(pan))[1];
+    const char *userptr = (const char *) panel_userptr(pan);
+    int num = (userptr && *userptr) ? userptr[1] : '?';
     int y, x;
 
     for (y = 0; y < getmaxy(win); y++) {
@@ -633,8 +636,8 @@ get_command(PANEL * px[MAX_PANELS + 1], char *buffer, int limit)
 
     if (log_in != 0) {
 	if (fgets(buffer, limit - 3, log_in) != 0) {
-	    length = strlen(buffer);
-	    while (length > 0 && isspace(buffer[length - 1]))
+	    length = (int) strlen(buffer);
+	    while (length > 0 && isspace(UChar(buffer[length - 1])))
 		buffer[--length] = '\0';
 	    waddstr(win, buffer);
 	} else {
@@ -663,22 +666,22 @@ get_command(PANEL * px[MAX_PANELS + 1], char *buffer, int limit)
 		    } else if (isdigit(UChar(c0))) {
 			wprintw(win, " %c", ch);
 			buffer[length++] = ' ';
-			buffer[length++] = c0 = ch;
+			buffer[length++] = (char) (c0 = ch);
 		    } else {
 			wprintw(win, "%c", ch);
-			buffer[length++] = c0 = ch;
+			buffer[length++] = (char) (c0 = ch);
 		    }
 		} else if (ok_digit(ch)) {
 		    if (isalpha(UChar(c0))) {
 			wprintw(win, "%c", ch);
-			buffer[length++] = c0 = ch;
+			buffer[length++] = (char) (c0 = ch);
 		    } else {
 			beep();
 		    }
 		} else if (ch == ' ') {
 		    if (isdigit(UChar(c0))) {
 			wprintw(win, "%c", ch);
-			buffer[length++] = c0 = ch;
+			buffer[length++] = (char) (c0 = ch);
 		    } else {
 			beep();
 		    }
@@ -714,7 +717,7 @@ demo_panels(InitPanel myInit, FillPanel myFill)
     memset(px, 0, sizeof(px));
 
     while (get_command(px, buffer, sizeof(buffer))) {
-	int limit = strlen(buffer);
+	int limit = (int) strlen(buffer);
 	for (itmp = 0; itmp < limit; itmp += 3) {
 	    do_panel(px, buffer + itmp, myFill);
 	}
@@ -759,7 +762,7 @@ main(int argc, char *argv[])
 
     setlocale(LC_ALL, "");
 
-    while ((c = getopt(argc, argv, "i:o:mwx")) != EOF) {
+    while ((c = getopt(argc, argv, "i:o:mwx")) != -1) {
 	switch (c) {
 	case 'i':
 	    log_in = fopen(optarg, "r");
