@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998-2002,2006 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2009,2010 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -41,7 +41,7 @@
  *
  *	Date: 05.Nov.90
  *
- * $Id: hanoi.c,v 1.25 2006/04/22 22:41:22 tom Exp $
+ * $Id: hanoi.c,v 1.29 2010/05/01 19:12:26 tom Exp $
  */
 
 #include <test.priv.h>
@@ -81,6 +81,7 @@ static int TileColour[] =
     COLOR_RED,			/* Length 19 */
 };
 static int NMoves = 0;
+static bool AutoFlag = FALSE;
 
 static void InitTiles(int NTiles);
 static void DisplayTiles(void);
@@ -95,7 +96,6 @@ int
 main(int argc, char **argv)
 {
     int NTiles, FromCol, ToCol;
-    bool AutoFlag = 0;
 
     setlocale(LC_ALL, "");
 
@@ -165,14 +165,14 @@ main(int argc, char **argv)
 	    if (GetMove(&FromCol, &ToCol))
 		break;
 	    if (InvalidMove(FromCol, ToCol)) {
-		mvaddstr(STATUSLINE, 0, "Invalid Move !!");
+		MvAddStr(STATUSLINE, 0, "Invalid Move !!");
 		refresh();
 		beep();
 		continue;
 	    }
 	    MakeMove(FromCol, ToCol);
 	    if (Solved(NTiles)) {
-		mvprintw(STATUSLINE, 0,
+		MvPrintw(STATUSLINE, 0,
 			 "Well Done !! You did it in %d moves", NMoves);
 		refresh();
 		sleep(5);
@@ -226,45 +226,48 @@ DisplayTiles(void)
     char TileBuf[BUFSIZ];
 
     erase();
-    mvaddstr(1, 24, "T O W E R S   O F   H A N O I");
-    mvaddstr(3, 34, "SJR 1990");
-    mvprintw(19, 5, "Moves : %d", NMoves);
-    attrset(A_REVERSE);
-    mvaddstr(BASELINE, 8,
+    MvAddStr(1, 24, "T O W E R S   O F   H A N O I");
+    MvAddStr(3, 34, "SJR 1990");
+    MvPrintw(19, 5, "Moves : %d", NMoves);
+    (void) attrset(A_REVERSE);
+    MvAddStr(BASELINE, 8,
 	     "                                                               ");
 
     for (Line = TOPLINE; Line < BASELINE; Line++) {
-	mvaddch(Line, LEFTPEG, ' ');
-	mvaddch(Line, MIDPEG, ' ');
-	mvaddch(Line, RIGHTPEG, ' ');
+	MvAddCh(Line, LEFTPEG, ' ');
+	MvAddCh(Line, MIDPEG, ' ');
+	MvAddCh(Line, RIGHTPEG, ' ');
     }
-    mvaddch(BASELINE, LEFTPEG, '1');
-    mvaddch(BASELINE, MIDPEG, '2');
-    mvaddch(BASELINE, RIGHTPEG, '3');
-    attrset(A_NORMAL);
+    MvAddCh(BASELINE, LEFTPEG, '1');
+    MvAddCh(BASELINE, MIDPEG, '2');
+    MvAddCh(BASELINE, RIGHTPEG, '3');
+    (void) attrset(A_NORMAL);
 
     /* Draw tiles */
     for (peg = 0; peg < NPEGS; peg++) {
 	for (SlotNo = 0; SlotNo < Pegs[peg].Count; SlotNo++) {
-	    memset(TileBuf, ' ', Pegs[peg].Length[SlotNo]);
-	    TileBuf[Pegs[peg].Length[SlotNo]] = '\0';
-	    if (has_colors())
-		attrset(COLOR_PAIR(LENTOIND(Pegs[peg].Length[SlotNo])));
-	    else
-		attrset(A_REVERSE);
-	    mvaddstr(BASELINE - (SlotNo + 1),
-		     (int) (PegPos[peg] - Pegs[peg].Length[SlotNo] / 2),
-		     TileBuf);
+	    unsigned len = Pegs[peg].Length[SlotNo];
+	    if (len < sizeof(TileBuf) - 1 && len < (unsigned) PegPos[peg]) {
+		memset(TileBuf, ' ', len);
+		TileBuf[len] = '\0';
+		if (has_colors())
+		    (void) attrset(COLOR_PAIR(LENTOIND(len)));
+		else
+		    (void) attrset(A_REVERSE);
+		MvAddStr(BASELINE - (SlotNo + 1),
+			 (int) (PegPos[peg] - len / 2),
+			 TileBuf);
+	    }
 	}
     }
-    attrset(A_NORMAL);
+    (void) attrset(A_NORMAL);
     refresh();
 }
 
 static int
 GetMove(int *From, int *To)
 {
-    mvaddstr(STATUSLINE, 0, "Next move ('q' to quit) from ");
+    MvAddStr(STATUSLINE, 0, "Next move ('q' to quit) from ");
     clrtoeol();
     refresh();
     if ((*From = getch()) == 'q')
@@ -278,7 +281,8 @@ GetMove(int *From, int *To)
 	return TRUE;
     *To -= ('0' + 1);
     refresh();
-    napms(500);
+    if (!AutoFlag)
+	napms(500);
 
     move(STATUSLINE, 0);
     clrtoeol();
