@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 2007 Free Software Foundation, Inc.                        *
+ * Copyright (c) 2007,2010 Free Software Foundation, Inc.                   *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -26,7 +26,9 @@
  * authorization.                                                           *
  ****************************************************************************/
 /*
- * $Id: inchs.c,v 1.7 2007/06/30 17:50:44 tom Exp $
+ * $Id: inchs.c,v 1.10 2010/05/01 19:13:46 tom Exp $
+ *
+ * Author: Thomas E Dickey
  */
 /*
        chtype inch(void);
@@ -100,10 +102,10 @@ test_inchs(int level, char **argv, WINDOW *chrwin, WINDOW *strwin)
 		break;
 	    }
 	}
+	fclose(fp);
     } else {
 	wprintw(txtwin, "Cannot open:\n%s", argv[1]);
     }
-    fclose(fp);
 
     while (!Quit(j = mvwgetch(txtwin, txt_y, txt_x))) {
 	switch (j) {
@@ -150,7 +152,7 @@ test_inchs(int level, char **argv, WINDOW *chrwin, WINDOW *strwin)
 	    break;
 	}
 
-	mvwprintw(chrwin, 0, 0, "char:");
+	MvWPrintw(chrwin, 0, 0, "char:");
 	wclrtoeol(chrwin);
 
 	if (txtwin != stdscr) {
@@ -188,7 +190,7 @@ test_inchs(int level, char **argv, WINDOW *chrwin, WINDOW *strwin)
 	}
 	wnoutrefresh(chrwin);
 
-	mvwprintw(strwin, 0, 0, "text:");
+	MvWPrintw(strwin, 0, 0, "text:");
 	wclrtobot(strwin);
 
 	limit = getmaxx(strwin) - 5;
@@ -196,38 +198,38 @@ test_inchs(int level, char **argv, WINDOW *chrwin, WINDOW *strwin)
 	if (txtwin != stdscr) {
 	    wmove(txtwin, txt_y, txt_x);
 	    if (winchstr(txtwin, text) != ERR) {
-		mvwaddchstr(strwin, 0, 5, text);
+		MvWAddChStr(strwin, 0, 5, text);
 	    }
 
 	    wmove(txtwin, txt_y, txt_x);
 	    if (winchnstr(txtwin, text, limit) != ERR) {
-		mvwaddchstr(strwin, 1, 5, text);
+		MvWAddChStr(strwin, 1, 5, text);
 	    }
 
 	    if (mvwinchstr(txtwin, txt_y, txt_x, text) != ERR) {
-		mvwaddchstr(strwin, 2, 5, text);
+		MvWAddChStr(strwin, 2, 5, text);
 	    }
 
 	    if (mvwinchnstr(txtwin, txt_y, txt_x, text, limit) != ERR) {
-		mvwaddchstr(strwin, 3, 5, text);
+		MvWAddChStr(strwin, 3, 5, text);
 	    }
 	} else {
 	    move(txt_y, txt_x);
 	    if (inchstr(text) != ERR) {
-		mvwaddchstr(strwin, 0, 5, text);
+		MvWAddChStr(strwin, 0, 5, text);
 	    }
 
 	    move(txt_y, txt_x);
 	    if (inchnstr(text, limit) != ERR) {
-		mvwaddchstr(strwin, 1, 5, text);
+		MvWAddChStr(strwin, 1, 5, text);
 	    }
 
 	    if (mvinchstr(txt_y, txt_x, text) != ERR) {
-		mvwaddchstr(strwin, 2, 5, text);
+		MvWAddChStr(strwin, 2, 5, text);
 	    }
 
 	    if (mvinchnstr(txt_y, txt_x, text, limit) != ERR) {
-		mvwaddchstr(strwin, 3, 5, text);
+		MvWAddChStr(strwin, 3, 5, text);
 	    }
 	}
 
@@ -266,5 +268,5 @@ main(int argc, char *argv[])
     test_inchs(1, argv, chrwin, strwin);
 
     endwin();
-    return EXIT_SUCCESS;
+    ExitProgram(EXIT_SUCCESS);
 }
