@@ -1,13 +1,40 @@
+/****************************************************************************
+ * Copyright (c) 1998-2004,2005 Free Software Foundation, Inc.              *
+ *                                                                          *
+ * Permission is hereby granted, free of charge, to any person obtaining a  *
+ * copy of this software and associated documentation files (the            *
+ * "Software"), to deal in the Software without restriction, including      *
+ * without limitation the rights to use, copy, modify, merge, publish,      *
+ * distribute, distribute with modifications, sublicense, and/or sell       *
+ * copies of the Software, and to permit persons to whom the Software is    *
+ * furnished to do so, subject to the following conditions:                 *
+ *                                                                          *
+ * The above copyright notice and this permission notice shall be included  *
+ * in all copies or substantial portions of the Software.                   *
+ *                                                                          *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
+ * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
+ * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
+ * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
+ * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
+ * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
+ *                                                                          *
+ * Except as contained in this notice, the name(s) of the above copyright   *
+ * holders shall not be used in advertising or otherwise to promote the     *
+ * sale, use or other dealings in this Software without prior written       *
+ * authorization.                                                           *
+ ****************************************************************************/
 /*
  * Test lower-right-hand corner access
  *
  * originally by Eric S. Raymond <esr@thyrsus.com>, written for animation
- * and resizing -TD
+ * and resizing -T.Dickey
  *
  * This can't be part of the ncurses test-program, because ncurses rips off the
  * bottom line to do labels.
  *
- * $Id: lrtest.c,v 1.19 2002/07/13 16:55:50 tom Exp $
+ * $Id: lrtest.c,v 1.21 2005/12/31 16:51:53 tom Exp $
  */
 
 #include <test.priv.h>
@@ -22,7 +49,7 @@ typedef struct {
  * really write to all positions properly.
  */
 static void
-show(MARK * m)
+show(MARK *m)
 {
     mvaddch(m->y, m->x, m->value);
     if (m->mode == 0) {		/* along the x-direction */
@@ -84,9 +111,12 @@ main(
 #endif
     move(LINES / 2 - 1, 4);
     if (!(has_ic()
+#if HAVE_SETUPTERM
     /* see PutCharLR() */
 	  || auto_right_margin
-	  || (enter_am_mode && exit_am_mode))) {
+	  || (enter_am_mode && exit_am_mode)
+#endif
+	)) {
 	addstr("Your terminal lacks the capabilities needed to address the\n");
 	move(LINES / 2, 4);
 	addstr("lower-right-hand corner of the screen.\n");
@@ -120,7 +150,7 @@ main(
 	    else if (ch == 'T')
 		trace(0);
 	    else if (ch == 't')
-		trace(TRACE_CALLS|TRACE_ICALLS|TRACE_UPDATE);
+		trace(TRACE_CALLS | TRACE_ICALLS | TRACE_UPDATE);
 #endif
 #ifdef KEY_RESIZE
 	    else if (ch == KEY_RESIZE) {
