@@ -1,7 +1,19 @@
-#include "test.priv.h"
+/* $Id: tclock.c,v 1.27 2010/05/01 18:47:19 tom Exp $ */
+
+#include <test.priv.h>
 
 #include <math.h>
-#include <time.h>
+
+#if TIME_WITH_SYS_TIME
+# include <sys/time.h>
+# include <time.h>
+#else
+# if HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif
 
 /*
   tclock - analog/digital clock for curses.
@@ -41,7 +53,7 @@
 static void
 plot(int x, int y, char col)
 {
-    mvaddch(y, x, (chtype) col);
+    MvAddCh(y, x, (chtype) col);
 }
 
 /* Draw a diagonal(arbitrary) line using Bresenham's alogrithm. */
@@ -55,7 +67,7 @@ dline(int pair, int from_x, int from_y, int x2, int y2, char ch)
     int d;
 
     if (has_colors())
-	attrset(COLOR_PAIR(pair));
+	(void) attrset(COLOR_PAIR(pair));
 
     dx = x2 - from_x;
     dy = y2 - from_y;
@@ -103,9 +115,7 @@ dline(int pair, int from_x, int from_y, int x2, int y2, char ch)
 }
 
 int
-main(
-	int argc GCC_UNUSED,
-	char *argv[]GCC_UNUSED)
+main(int argc GCC_UNUSED, char *argv[]GCC_UNUSED)
 {
     int i, cx, cy;
     double cr, mradius, hradius, mangle, hangle;
@@ -115,10 +125,18 @@ main(
     int sdx, sdy;
     int ch;
     int lastbeep = -1;
+    bool odd = FALSE;
     time_t tim;
     struct tm *t;
     char szChar[10];
+    char *text;
     int my_bg = COLOR_BLACK;
+#if HAVE_GETTIMEOFDAY
+    struct timeval current;
+    double fraction = 0.0;
+#endif
+
+    setlocale(LC_ALL, "");
 
     initscr();
     noecho();
@@ -135,6 +153,7 @@ main(
 	init_pair(1, COLOR_RED, my_bg);
 	init_pair(2, COLOR_MAGENTA, my_bg);
 	init_pair(3, COLOR_GREEN, my_bg);
+	init_pair(4, COLOR_WHITE, COLOR_BLUE);
     }
 #ifdef KEY_RESIZE
     keypad(stdscr, TRUE);
@@ -156,10 +175,10 @@ main(
 	sdy = A2Y(sangle, sradius);
 	sprintf(szChar, "%d", i + 1);
 
-	mvaddstr(cy - sdy, cx + sdx, szChar);
+	MvAddStr(cy - sdy, cx + sdx, szChar);
     }
 
-    mvaddstr(0, 0, "ASCII Clock by Howard Jones (ha.jones@ic.ac.uk),1994");
+    MvAddStr(0, 0, "ASCII Clock by Howard Jones (ha.jones@ic.ac.uk),1994");
 
     sradius = (4 * sradius) / 5;
     for (;;) {
@@ -172,7 +191,7 @@ main(
 	if (hours > 12.0)
 	    hours -= 12.0;
 
-	mangle = ((t->tm_min) * (2 * PI) / 60.0);
+	mangle = ((t->tm_min + (t->tm_sec / 60.0)) * (2 * PI) / 60.0);
 	mdx = A2X(mangle, mradius);
 	mdy = A2Y(mangle, mradius);
 
@@ -180,29 +199,38 @@ main(
 	hdx = A2X(hangle, hradius);
 	hdy = A2Y(hangle, hradius);
 
-	sangle = ((t->tm_sec) * (2.0 * PI) / 60.0);
+#if HAVE_GETTIMEOFDAY
+	gettimeofday(&current, 0);
+	fraction = (current.tv_usec / 1.0e6);
+#endif
+	sangle = ((t->tm_sec + fraction) * (2.0 * PI) / 60.0);
 	sdx = A2X(sangle, sradius);
 	sdy = A2Y(sangle, sradius);
 
 	dline(3, cx, cy, cx + mdx, cy - mdy, '#');
 
-	attrset(A_REVERSE);
+	(void) attrset(A_REVERSE);
 	dline(2, cx, cy, cx + hdx, cy - hdy, '.');
 	attroff(A_REVERSE);
 
 	if (has_colors())
-	    attrset(COLOR_PAIR(1));
+	    (void) attrset(COLOR_PAIR(1));
 
-	plot(cx + sdx, cy - sdy, 'O');
+	dline(1, cx, cy, cx + sdx, cy - sdy, 'O');
 
 	if (has_colors())
-	    attrset(COLOR_PAIR(0));
+	    (void) attrset(COLOR_PAIR(0));
 
-	mvaddstr(LINES - 2, 0, ctime(&tim));
+	text = ctime(&tim);
+	MvPrintw(2, 0, "%.*s", (int) (strlen(text) - 1), text);
 	refresh();
 	if ((t->tm_sec % 5) == 0
 	    && t->tm_sec != lastbeep) {
 	    lastbeep = t->tm_sec;
+	    if (has_colors()) {
+		odd = !odd;
+		bkgd((chtype) (odd ? COLOR_PAIR(4) : COLOR_PAIR(0)));
+	    }
 	    beep();
 	}
 
@@ -218,9 +246,9 @@ main(
 	    break;
 	}
 
-	plot(cx + sdx, cy - sdy, ' ');
 	dline(0, cx, cy, cx + hdx, cy - hdy, ' ');
 	dline(0, cx, cy, cx + mdx, cy - mdy, ' ');
+	dline(0, cx, cy, cx + sdx, cy - sdy, ' ');
 
     }
 
