@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998-2006,2007 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2009,2010 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -50,7 +50,7 @@
  * scroll operation worked, and the refresh() code only had to do a
  * partial repaint.
  *
- * $Id: view.c,v 1.65 2007/06/30 16:40:04 tom Exp $
+ * $Id: view.c,v 1.79 2010/05/01 21:56:10 tom Exp $
  */
 
 #include <test.priv.h>
@@ -62,8 +62,10 @@
 #if HAVE_TERMIOS_H
 # include <termios.h>
 #else
+#if !defined(__MINGW32__)
 # include <sgtty.h>
 #endif
+#endif
 
 #if !defined(sun) || !HAVE_TERMIOS_H
 # if HAVE_SYS_IOCTL_H
@@ -79,6 +81,23 @@
 #include <sys/ptem.h>
 #endif
 
+#if USE_WIDEC_SUPPORT
+#if HAVE_MBTOWC && HAVE_MBLEN
+#define reset_mbytes(state) IGNORE_RC(mblen(NULL, 0)), IGNORE_RC(mbtowc(NULL, NULL, 0))
+#define count_mbytes(buffer,length,state) mblen(buffer,length)
+#define check_mbytes(wch,buffer,length,state) \
+	(int) mbtowc(&wch, buffer, length)
+#define state_unused
+#elif HAVE_MBRTOWC && HAVE_MBRLEN
+#define reset_mbytes(state) init_mb(state)
+#define count_mbytes(buffer,length,state) mbrlen(buffer,length,&state)
+#define check_mbytes(wch,buffer,length,state) \
+	(int) mbrtowc(&wch, buffer, length, &state)
+#else
+make an error
+#endif
+#endif				/* USE_WIDEC_SUPPORT */
+
 static RETSIGTYPE finish(int sig) GCC_NORETURN;
 static void show_all(const char *tag);
 
@@ -132,11 +151,19 @@ ch_len(NCURSES_CH_T * src)
 {
     int result = 0;
 #if USE_WIDEC_SUPPORT
+    int count;
 #endif
 
 #if USE_WIDEC_SUPPORT
-    while (getcchar(src++, NULL, NULL, NULL, NULL) > 0)
-	result++;
+    for (;;) {
+	TEST_CCHAR(src, count, {
+	    ++result;
+	    ++src;
+	}
+	, {
+	    break;
+	})
+    }
 #else
     while (*src++)
 	result++;
@@ -158,17 +185,19 @@ ch_dup(char *src)
     wchar_t wstr[CCHARW_MAX + 1];
     wchar_t wch;
     int l = 0;
-    mbstate_t state;
     size_t rc;
     int width;
+#ifndef state_unused
+    mbstate_t state;
 #endif
+#endif /* USE_WIDEC_SUPPORT */
 
 #if USE_WIDEC_SUPPORT
-    memset(&state, 0, sizeof(state));
+    reset_mbytes(state);
 #endif
     for (j = k = 0; j < len; j++) {
 #if USE_WIDEC_SUPPORT
-	rc = mbrtowc(&wch, src + j, len - j, &state);
+	rc = check_mbytes(wch, src + j, len - j, state);
 	if (rc == (size_t) -1 || rc == (size_t) -2)
 	    break;
 	j += rc - 1;
@@ -229,7 +258,7 @@ main(int argc, char *argv[])
     (void) signal(SIGINT, finish);	/* arrange interrupts to terminate */
 #endif
 
-    while ((i = getopt(argc, argv, "cin:rtT:")) != EOF) {
+    while ((i = getopt(argc, argv, "cin:rtT:")) != -1) {
 	switch (i) {
 	case 'c':
 	    try_color = TRUE;
@@ -238,7 +267,8 @@ main(int argc, char *argv[])
 	    CATCHALL(SIG_IGN);
 	    break;
 	case 'n':
-	    if ((MAXLINES = atoi(optarg)) < 1)
+	    if ((MAXLINES = atoi(optarg)) < 1 ||
+		(MAXLINES + 2) <= 1)
 		usage();
 	    break;
 #if CAN_RESIZE
@@ -261,9 +291,11 @@ main(int argc, char *argv[])
     if (optind + 1 != argc)
 	usage();
 
-    if ((vec_lines = typeMalloc(NCURSES_CH_T *, MAXLINES + 2)) == 0)
+    if ((vec_lines = typeCalloc(NCURSES_CH_T *, MAXLINES + 2)) == 0)
 	usage();
 
+    assert(vec_lines != 0);
+
     fname = argv[optind];
     if ((fp = fopen(fname, "r")) == 0) {
 	perror(fname);
@@ -335,7 +367,6 @@ main(int argc, char *argv[])
 	if (!got_number)
 	    show_all(my_label);
 
-	n = 0;
 	for (;;) {
 #if CAN_RESIZE
 	    if (interrupted) {
@@ -348,7 +379,7 @@ main(int argc, char *argv[])
 	    waiting = FALSE;
 	    if ((c < 127) && isdigit(c)) {
 		if (!got_number) {
-		    mvprintw(0, 0, "Count: ");
+		    MvPrintw(0, 0, "Count: ");
 		    clrtoeol();
 		}
 		addch(UChar(c));
@@ -470,12 +501,11 @@ finish(int sig)
 
 #if CAN_RESIZE
 /*
- * This uses functions that are "unsafe", but it seems to work on SunOS and
- * Linux.  Usually:  the "unsafe" refers to the functions that POSIX lists
- * which may be called from a signal handler.  Those do not include buffered
- * I/O, which is used for instance in wrefresh().  To be really portable, you
- * should use the KEY_RESIZE return (which relies on ncurses' sigwinch
- * handler).
+ * This uses functions that are "unsafe", but it seems to work on SunOS. 
+ * Usually: the "unsafe" refers to the functions that POSIX lists which may be
+ * called from a signal handler.  Those do not include buffered I/O, which is
+ * used for instance in wrefresh().  To be really portable, you should use the
+ * KEY_RESIZE return (which relies on ncurses' sigwinch handler).
  *
  * The 'wrefresh(curscr)' is needed to force the refresh to start from the top
  * of the screen -- some xterms mangle the bitmap while resizing.
@@ -488,7 +518,7 @@ adjust(int sig)
 
 	if (ioctl(fileno(stdout), TIOCGWINSZ, &size) == 0) {
 	    resize_term(size.ws_row, size.ws_col);
-	    wrefresh(curscr);	/* Linux needs this */
+	    wrefresh(curscr);
 	    show_all(sig ? "SIGWINCH" : "interrupt");
 	}
 	interrupted = FALSE;
@@ -508,10 +538,12 @@ show_all(const char *tag)
     time_t this_time;
 
 #if CAN_RESIZE
-    sprintf(temp, "%s (%3dx%3d) col %d ", tag, LINES, COLS, shift);
+    sprintf(temp, "%.20s (%3dx%3d) col %d ", tag, LINES, COLS, shift);
     i = strlen(temp);
-    sprintf(temp + i, "view %.*s", (int) (sizeof(temp) - 7 - i), fname);
+    if ((i + 7) < (int) sizeof(temp))
+	sprintf(temp + i, "view %.*s", (int) (sizeof(temp) - 7 - i), fname);
 #else
+    (void) tag;
     sprintf(temp, "view %.*s", (int) sizeof(temp) - 7, fname);
 #endif
     move(0, 0);
